import os

import mutagen

from django.conf import settings
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.timezone import now
from django.utils.translation import ugettext_lazy as _

REMOTE_BASE_PATH = '/srv/soma/nonstop/'
LOCAL_BASE_PATH = '/media/nonstop/'

TRANCHE_SLUG_DIR_MAPPING = {
    'acouphene': 'Acouphene',
    'biodiversite': 'Biodiversite',
    'l-heure-de-pointe': 'Heure_de_pointe',
    'hop-bop-co': 'Hop_Bop_and_co',
    'la-panique': 'la_panique',
    'le-mange-disque': 'Mange_Disque',
    'matin-tranquille': 'Matins_tranquilles',
    'reveries': 'Reveries',
    'up-beat-tempo': 'Up_Beat_Tempo',
}

class Artist(models.Model):
    name = models.CharField(_('Name'), max_length=255)

    class Meta:
        ordering = ['name']

    def get_absolute_url(self):
        return reverse('artist-view', kwargs={'pk': self.id})

    def recent_diffusions(self):
        return SomaLogLine.objects.filter(filepath__track__artist=self
                ).exclude(on_air=False).order_by('-play_timestamp')


class Album(models.Model):
    name = models.CharField(_('Name'), max_length=255)


LANGUAGES = [
    ('en', _('English')),
    ('fr', _('French')),
    ('nl', _('Dutch'))
]

class Track(models.Model):
    title = models.CharField(_('Title'), max_length=255)
    artist = models.ForeignKey(Artist, null=True)
    album = models.ForeignKey(Album, null=True)
    instru = models.BooleanField(_('Instru'), default=False)
    language = models.CharField(max_length=3,
            choices=LANGUAGES, blank=True)
    sabam = models.BooleanField('SABAM', default=True)
    cfwb = models.BooleanField('CFWB', default=False)
    nonstop_zones = models.ManyToManyField('emissions.Nonstop', blank=True)

    creation_timestamp = models.DateTimeField(auto_now_add=True, null=True)
    added_to_nonstop_timestamp = models.DateTimeField(null=True)
    uploader = models.ForeignKey(settings.AUTH_USER_MODEL, null=True)
    duration = models.DurationField(_('Duration'), null=True)

    class Meta:
        ordering = ['creation_timestamp']

    def get_absolute_url(self):
        return reverse('track-view', kwargs={'pk': self.id})

    def recent_diffusions(self):
        return SomaLogLine.objects.filter(filepath__track=self
                ).exclude(on_air=False).order_by('-play_timestamp')

    def file_path(self):
        nfile = None
        for nfile in self.nonstopfile_set.all().order_by('creation_timestamp'):
            if os.path.exists(nfile.get_local_filepath()):
                return nfile.get_local_filepath()
        if nfile:
           return nfile.get_local_filepath()
        return None

    def file_exists(self):
        file_path = self.file_path()
        if not file_path:
            return False
        try:
            return os.path.exists(file_path)
        except AttributeError:
            return False

    def sync_nonstop_zones(self):
        current_zones = self.nonstop_zones.all()
        if current_zones.count():
            if not self.added_to_nonstop_timestamp:
                self.added_to_nonstop_timestamp = now()
                self.save()
        else:
            self.added_to_nonstop_timestamp = None
            self.save()

        if not self.file_exists():
            return
        nonstop_file = self.nonstopfile_set.order_by('creation_timestamp').last()
        filename = nonstop_file.filename
        from emissions.models import Nonstop

        for zone in Nonstop.objects.all():
            if not zone.slug in TRANCHE_SLUG_DIR_MAPPING:
                continue
            zone_dir = TRANCHE_SLUG_DIR_MAPPING[zone.slug]
            zone_path = os.path.join(LOCAL_BASE_PATH, 'Tranches', zone_dir, filename)
            if zone in current_zones:
                if not os.path.exists(zone_path):
                    os.symlink(os.path.join('..', '..', nonstop_file.short), zone_path)
            else:
                if os.path.exists(zone_path):
                    os.unlink(zone_path)


class NonstopFile(models.Model):
    filepath = models.CharField(_('Filepath'), max_length=255)
    filename = models.CharField(_('Filename'), max_length=255, null=True)
    creation_timestamp = models.DateTimeField(auto_now_add=True, null=True)
    track = models.ForeignKey(Track, null=True)

    @property
    def short(self):
        return self.filepath[len(REMOTE_BASE_PATH):]

    def set_track_filepath(self, filepath):
        self.filepath = os.path.join(REMOTE_BASE_PATH, 'tracks', filepath)
        self.filename = os.path.basename(filepath)

    def get_local_filepath(self):
        if not self.short:
            return None
        return os.path.join(LOCAL_BASE_PATH, self.short)


class SomaLogLine(models.Model):
    class Meta:
        verbose_name = _('Soma log line')
        verbose_name_plural = _('Soma log lines')
        ordering = ['play_timestamp']

    filepath = models.ForeignKey(NonstopFile)
    play_timestamp = models.DateTimeField()
    on_air = models.NullBooleanField('On Air')


class Jingle(models.Model):
    class Meta:
        verbose_name = _('Jingle')
        verbose_name_plural = _('Jingles')
        ordering = ['label']

    label = models.CharField(_('Label'), max_length=100)
    filepath = models.CharField(_('File Path'), max_length=255)
    duration = models.DurationField(_('Duration'), null=True, blank=True)
    default_for_initial_diffusions = models.BooleanField(_('Default for initial diffusions'), default=False)
    default_for_reruns = models.BooleanField(_('Default for reruns'), default=False)
    default_for_streams = models.BooleanField(_('Default for streams'), default=False)

    def __str__(self):
        return self.label

    def save(self, **kwargs):
        for attr in ('default_for_initial_diffusions', 'default_for_reruns', 'default_for_streams'):
            if getattr(self, attr):
                Jingle.objects.all().update(**{attr: False})
        return super(Jingle, self).save(**kwargs)


class Stream(models.Model):
    class Meta:
        verbose_name = _('Stream')
        verbose_name_plural = _('Streams')
        ordering = ['label']

    label = models.CharField(_('Label'), max_length=100)
    url = models.URLField(_('URL'), max_length=255)

    def __str__(self):
        return self.label


class StreamedDiffusion(models.Model):
    class Meta:
        verbose_name = _('Streamed diffusion')
        verbose_name_plural = _('Streamed diffusions')

    diffusion = models.ForeignKey('emissions.Diffusion')
    jingle = models.ForeignKey(Jingle, null=True, blank=True)
    stream = models.ForeignKey(Stream)
